

class Action( object ):
    def __init__( self, name ):
        self.name   = name
        self.parent = None
        self.menu   = None

    def run( self ):
        return self

    def init( self ):
        pass


class Exit( Action ):
    def __init__( self ):
        super( Exit, self ).__init__( "Exit" )

    def run( self ):
        return self


class Back( Action ):
    def __init__( self ):
        super( Back, self ).__init__( "Back" )


class Cmd( Action ):
    def __init__( self, name, cmd ):
        super( Cmd, self ).__init__( name )
        self.cmd = cmd

    def run( self ):
        from os import system
        system( self.cmd )

        if not self.menu.is_automated: raw_input( "\nFinished...." )



import Screen
from . import Page


class Main( Page ):
    def __init__( self, title, chars = None ):
        self.title         = title
        self.chars         = chars
        self.current       = [ self ]
        self.current_title = ""
        self.width, self.height = Screen.size()
        self.is_automated  = False

        if self.chars  is None: self.chars = [ "=", "-", ") " ]

        super( Main, self ).__init__( "Main Menu", menu = self, parent = self )

    def header( self ):
        Screen.clear()
        self.width, self.height = Screen.size()
        print self.chars[0] * self.width

        spaces = (self.width - len(self.title)) / 2
        print "%s%s" % (" " * spaces, self.title)

        print self.chars[0] * self.width

        title = ""
        if len( self.current ) == 1: title = self.current[0].name
        else:
            for p in self.current:
                if p.name == "Main Menu": continue
                elif title != "": title += "."
                title += p.name.lower().replace( " ", "_" )

        print
        print title
        print self.chars[1] * ( self.width / 4 )
        self.current_title = title

    def cli( self, commands = None ):
        from . import Page, Action

        # If a list of commands is provided as a string, convert them to an array
        if commands is not None:
            self.is_automated = True
            if isinstance( commands, basestring ): commands = list( commands )
        else:
            self.is_automated = False

        while True:
            self.header()
            self.current[-1].choices()
            print self.chars[1] * (self.width / 4)

            try:
                if commands is not None: choice = commands.pop()
                else                   : choice = raw_input( "\nChoice: " )

                if choice.lower() == "q": break
                rtn = self.current[-1].run( int( choice ) )

                if   isinstance( rtn, Page        ): self.current.append( rtn )
                elif isinstance( rtn, Action.Back ): self.current.pop()
                elif isinstance( rtn, Action.Exit ): break
            except ValueError:
                print "Incorrect value, try again...."
                if commands is None:  raw_input( "" )
            except KeyboardInterrupt:
                print
                break



from sys import stdout


def size():
    import fcntl, termios, struct
    h, w, hp, wp = struct.unpack(
        "HHHH",
        fcntl.ioctl(0, termios.TIOCGWINSZ, struct.pack("HHHH", 0, 0, 0, 0))
    )
    return w, h


def clear():
    stdout.write("\033c")



"""
pyman
-------------

A CLI for managing python projects
"""
from setuptools import setup


setup(
    name='pyman',
    version='0.1.0a1',
    url='http://www.mlit.net.au/pyman/',
    license='MIT',
    author='Mark Pittaway',
    author_email='mark.pittaway@mlit.net.au',
    description='A CLI for managing python projects',
    long_description=__doc__,
    packages=['pyman'],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=[],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Environment :: Console',
        'Topic :: Software Development :: Testing',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: User Interfaces',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Operating System :: POSIX :: Linux'
    ]
)





class Page( object ):
    def __init__( self, name, parent = None, menu = None ):
        self.name    = name
        self.parent  = parent
        self.menu    = menu
        self.actions = []

    def init( self ):
        pass

    def choices( self ):
        i = 1
        for a in self.actions:
            print "%d%s%s" % ( i, self.menu.chars[2], a.name )
            i += 1

    def add( self, actions ):
        for a in actions:
            a.parent = self
            a.menu   = self.menu
            a.init()
            self.actions.append( a )

    def add_action( self, action ):
        action.parent = self
        action.menu   = self.menu
        action.init()
        self.actions.append( action )

    def run( self, index ):
        if index <= 0 or index > len( self.actions ):
            raw_input( "Value out of range, try again...." )
            return

        action = self.actions[ index - 1 ]

        self.menu.current.append( action )
        self.menu.header()
        self.menu.current.pop()

        if isinstance( action, Page ): return action
        return action.run()
